using System;
using System.Collections.Specialized;
using System.Reflection;
using System.Xml.Serialization;

using Microsoft.Win32;

using GBPVR.Public;

using RecorderHelper;

namespace ExternalRecorder
{
	/// <summary>
	/// Summary description for ExternalRecorderConfig.
	/// </summary>
	[XmlType("captureSource")]
	public class ExternalRecorderConfig : IRecordingSourceConfiguration
	{

		#region Constructor / Destructor

		public ExternalRecorderConfig()
		{
			// constructor
			string version = Assembly.GetExecutingAssembly().GetName().Version.ToString();
			Logger.Verbose("ExternalRecorderConfig[" + this.GetHashCode() + "]: ExternalRecorderConfig(version=" + version + ")");
		}

		public ExternalRecorderConfig(ExternalRecorder recordingSource)
		{
			// constructor
			string version = Assembly.GetExecutingAssembly().GetName().Version.ToString();
			Logger.Verbose("ExternalRecorderConfig[" + this.GetHashCode() + "]: ExternalRecorderConfig(version=" + version + ", recordingSource=[" + recordingSource.GetHashCode() + "])");
			this.recordingSource = recordingSource;
		}

		#endregion

		#region Private Variables

		[XmlAttribute("oid"),XmlHashKey()]
		internal int oid;

		internal string executable;
		internal string recordingParameters;
		internal string previewParameters;

		internal bool threeDigit;
		internal bool zeroPad;

		[NonSerialized()]
		internal ExternalRecorder recordingSource;

		#endregion

		#region IRecordingSourceConfiguration Members

		public void Save(System.Data.OleDb.OleDbConnection connection, int captureSourceOID)
		{
			// save the plugin settings
			Logger.Verbose("ExternalRecorderConfig[" + this.GetHashCode() + "]: Save(" + captureSourceOID + ")");
			Config.getConfig().Save(captureSourceOID, this);
		}

		public void Delete(System.Data.OleDb.OleDbConnection connection, int captureSourceOID)
		{
			// delete the plugin settings from the database
			Logger.Verbose("ExternalRecorderConfig[" + this.GetHashCode() + "]: Delete(" + captureSourceOID + ")");
			Config.getConfig().Delete(captureSourceOID);
		}

		public void Load(int captureSourceOID)
		{
			// load the plugin settings
			Logger.Verbose("ExternalRecorderConfig[" + this.GetHashCode() + "]: Load(" + captureSourceOID + ")");
			this.oid = captureSourceOID;
			Config.getConfig().Load(captureSourceOID, this);
			recordingSources[captureSourceOID] = this;
		}

		public string getName()
		{
			// return the display name for this plugin
			return "External Recorder Plugin";
		}

		public string getConfigurationFormClassName()
		{
			// Return the name of the class which can be used to set up the configration for this plugin			
			// The form must have a constructor which takes a single int argument (which is the capture source OID)
			return typeof(RecorderConfig).FullName;
		}

		public IRecordingSource getRecordingSource()
		{
			if (recordingSource == null)
			{
				recordingSource = new ExternalRecorder(this);
                recordingSource.config = this;				
			}
			return recordingSource;
		}

		#endregion

		#region Singleton Implementation

		/// <summary> the recording source used for this configuration. </summary>
		[NonSerialized()]
		private static HybridDictionary	recordingSources = new HybridDictionary();

		internal static ExternalRecorderConfig getConfigFromMemory(int captureSourceOID)
		{
			ExternalRecorderConfig config = (ExternalRecorderConfig)recordingSources[captureSourceOID];
			Logger.Verbose("ExternalRecorderConfig: getConfigFromMemory(" + captureSourceOID + "), return [" + (config != null ? config.GetHashCode().ToString() : "null") + "]");
			return config;
		}

		#endregion

	}
}
